from .__version__ import __author__, __copyright__, __email__, __license__, __version__
from ._common import get_locals, get_providers
from ._tblfaker import TableFaker


__all__ = (
    "__author__",
    "__copyright__",
    "__email__",
    "__license__",
    "__version__",
    "get_locals",
    "get_providers",
    "TableFaker",
)
